@@ -18,9 +18,9 @@
 # limitations under the License.
 
 try:
-    import asyncio
+    from inspect import iscoroutinefunction
 except ImportError:
-    asyncio = None
+    iscoroutinefunction = None
 
 try:
     import tornado
@@ -96,7 +96,7 @@ def retry(*dargs, **dkw):
         return retry()(dargs[0])
     else:
         def wrap(f):
-            if asyncio and asyncio.iscoroutinefunction(f):
+            if iscoroutinefunction is not None and iscoroutinefunction(f):
                 r = AsyncRetrying(*dargs, **dkw)
             elif tornado and hasattr(tornado.gen, 'is_coroutine_function') \
                     and tornado.gen.is_coroutine_function(f):
@@ -161,6 +161,7 @@ class RetryError(Exception):
 
     def __init__(self, last_attempt):
         self.last_attempt = last_attempt
+        super(RetryError, self).__init__(last_attempt)
 
     def reraise(self):
         if self.last_attempt.failed:
@@ -171,6 +172,24 @@ class RetryError(Exception):
         return "{0}[{1}]".format(self.__class__.__name__, self.last_attempt)
 
 
+class AttemptManager(object):
+    """Manage attempt context."""
+
+    def __init__(self, retry_state):
+        self.retry_state = retry_state
+
+    def __enter__(self):
+        pass
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if isinstance(exc_value, BaseException):
+            self.retry_state.set_exception((exc_type, exc_value, traceback))
+            return True  # Swallow exception.
+        else:
+            # We don't have the result, actually.
+            self.retry_state.set_result(None)
+
+
 class BaseRetrying(object):
 
     def __init__(self,
@@ -345,6 +364,20 @@ class BaseRetrying(object):
 
         return DoSleep(sleep)
 
+    def __iter__(self):
+        self.begin(None)
+
+        retry_state = RetryCallState(self, fn=None, args=(), kwargs={})
+        while True:
+            do = self.iter(retry_state=retry_state)
+            if isinstance(do, DoAttempt):
+                yield AttemptManager(retry_state=retry_state)
+            elif isinstance(do, DoSleep):
+                retry_state.prepare_for_next_attempt()
+                self.sleep(do)
+            else:
+                break
+
 
 class Retrying(BaseRetrying):
     """Retrying controller."""
@@ -446,7 +479,7 @@ class RetryCallState(object):
         self.outcome, self.outcome_timestamp = fut, ts
 
 
-if asyncio:
+if iscoroutinefunction:
     from tenacity._asyncio import AsyncRetrying
 
 if tornado:
